@@ -56,7 +56,7 @@ int main_crop(int argc, char* argv[])
 
 	unmap_cfl(N, in_dims, in_data);
 	unmap_cfl(N, out_dims, out_data);
-	exit(0);
+	return 0;
 }
 
 
