@@ -13,7 +13,6 @@ struct noir_conf_s {
 
 	unsigned int iter;
 	_Bool rvc;
-	_Bool usegpu;
 	_Bool noncart;
 	float alpha;
 	float alpha_min;
@@ -24,9 +23,20 @@ struct noir_conf_s {
 	_Bool sms;
 };
 
+
+
 extern const struct noir_conf_s noir_defaults;
 
-extern void noir_recon(const struct noir_conf_s* conf, const long dims[DIMS], _Complex float* img, _Complex float* sens, const _Complex float* ref, const _Complex float* pattern, const _Complex float* mask, const _Complex float* kspace_data);
+extern void noir_recon(const struct noir_conf_s* conf,
+		       const long dims[DIMS],
+		       _Complex float* img,
+		       _Complex float* sens,
+		       _Complex float* ksens,
+		       const _Complex float* ref,
+		       const _Complex float* pattern,
+		       const _Complex float* mask,
+		       const _Complex float* kspace_data);
+
 
 #include "misc/cppwrap.h"
 
