@@ -110,30 +110,13 @@ static complex float* compute_linphases(int N, long lph_dims[N + 1], unsigned lo
 }
 
 
-static void compute_kern(unsigned int N, const long krn_dims[N], complex float* krn,
+
+static void compute_kern_basis(unsigned int N, unsigned int flags, const long pos[N],
+				const long krn_dims[N], complex float* krn,
 				const long bas_dims[N], const complex float* basis,
 				const long wgh_dims[N], const complex float* weights)
 {
-	if (NULL == basis) {
-
-		md_zfill(N, krn_dims, krn, 1.);
-
-		if (NULL != weights) {
-
-			long krn_strs[N];
-			md_calc_strides(N, krn_strs, krn_dims, CFL_SIZE);
-
-			long wgh_strs[N];
-			md_calc_strides(N, wgh_strs, wgh_dims, CFL_SIZE);
-
-			md_zmul2(N, krn_dims, krn_strs, krn, krn_strs, krn, wgh_strs, weights);
-			md_zmulc2(N, krn_dims, krn_strs, krn, krn_strs, krn, wgh_strs, weights);
-		}
-
-		return;
-	}
-
-	assert(1 == krn_dims[N - 1]);
+// 	assert(1 == krn_dims[N - 1]);
 	assert(1 == wgh_dims[N - 1]);
 	assert(1 == bas_dims[N - 1]);
 
@@ -169,6 +152,8 @@ static void compute_kern(unsigned int N, const long krn_dims[N], complex float*
 	wgT_strs[N - 1] = wgh_strs[5];
 	wgT_strs[5] = 0;
 
+	debug_printf(DP_DEBUG1, "Allocating %ld\n", md_calc_size(N, max_dims));
+
 	complex float* tmp = md_alloc(N, max_dims, CFL_SIZE);
 
 	md_copy2(N, max_dims, max_strs, tmp, baT_strs, basis, CFL_SIZE);
@@ -188,46 +173,174 @@ static void compute_kern(unsigned int N, const long krn_dims[N], complex float*
 	long ma2_dims[N];
 	md_tenmul_dims(N, ma2_dims, krn_dims, max_dims, baT_dims);
 
-	md_ztenmulc2(N, ma2_dims, krn_strs, krn, max_strs, tmp, baT_strs, basis);
+	long ma3_dims[N];
+	md_select_dims(N, flags, ma3_dims, ma2_dims);
 
-	md_zsmul(N, krn_dims, krn, krn, (double)bas_dims[6]);	// FIXME: Why?
+	long tmp_off = md_calc_offset(N, max_strs, pos);
+	long bas_off = md_calc_offset(N, baT_strs, pos);
+
+	md_zsmul(N, max_dims, tmp, tmp, (double)bas_dims[6]);	// FIXME: Why?
+
+	md_ztenmulc2(N, ma3_dims, krn_strs, krn, 
+			max_strs, (void*)tmp + tmp_off,
+			baT_strs, (void*)basis + bas_off);
 
 	md_free(tmp);
 }
 
 
-complex float* compute_psf(unsigned int N, const long img2_dims[N], const long trj_dims[N], const complex float* traj,
-				const long bas_dims[N], const complex float* basis,
-				const long wgh_dims[N], const complex float* weights, bool periodic)
-{
-	long ksp_dims1[N];
 
-	md_copy_dims(N, ksp_dims1, img2_dims);
-	md_select_dims(3, ~MD_BIT(0), ksp_dims1, trj_dims);
+static void compute_kern(unsigned int N, unsigned int flags, const long pos[N],
+				const long krn_dims[N], complex float* krn,
+				const long bas_dims[N], const complex float* basis,
+				const long wgh_dims[N], const complex float* weights)
+{
+	if (NULL != basis)
+		return compute_kern_basis(N, flags, pos, krn_dims, krn, bas_dims, basis, wgh_dims, weights);
+
+	assert(~0u == flags);
+
+	md_zfill(N, krn_dims, krn, 1.);
+
+	if (NULL != weights) {
+
+		long krn_strs[N];
+		md_calc_strides(N, krn_strs, krn_dims, CFL_SIZE);
+
+		long wgh_strs[N];
+		md_calc_strides(N, wgh_strs, wgh_dims, CFL_SIZE);
+
+		md_zmul2(N, krn_dims, krn_strs, krn, krn_strs, krn, wgh_strs, weights);
+		md_zmulc2(N, krn_dims, krn_strs, krn, krn_strs, krn, wgh_strs, weights);
+	}
+
+	return;
+}
+
+
+
+complex float* compute_psf(unsigned int N, const long img_dims[N], const long trj_dims[N], const complex float* traj,
+				const long bas_dims[N], const complex float* basis,
+				const long wgh_dims[N], const complex float* weights,
+				bool periodic, bool lowmem)
+{
+	long img2_dims[N + 1];
+	md_copy_dims(N, img2_dims, img_dims);
+	img2_dims[N] = 1;
+
+	long trj2_dims[N + 1];
+	md_copy_dims(N, trj2_dims, trj_dims);
+	trj2_dims[N] = 1;
+
+	long bas2_dims[N + 1];
+	md_copy_dims(N, bas2_dims, bas_dims);
+	bas2_dims[N] = 1;
+
+	long wgh2_dims[N + 1];
+	md_copy_dims(N, wgh2_dims, wgh_dims);
+	wgh2_dims[N] = 1;
+
+	N++;
+
+	long ksp2_dims[N];
+	md_copy_dims(N, ksp2_dims, img2_dims);
+	md_select_dims(3, ~MD_BIT(0), ksp2_dims, trj2_dims);
+
+	if (NULL != basis) {
+
+		assert(1 == trj2_dims[6]);
+		ksp2_dims[N - 1] = trj2_dims[5];
+		trj2_dims[N - 1] = trj2_dims[5];
+		trj2_dims[5] = 1;	// FIXME copy?
+	}
 
 	struct nufft_conf_s conf = nufft_conf_defaults;
 	conf.periodic = periodic;
 	conf.toeplitz = false;	// avoid infinite loop
 
-	struct linop_s* op2 = nufft_create(N, ksp_dims1, img2_dims, trj_dims, traj, NULL, conf);
 
-	complex float* ones = md_alloc(N, ksp_dims1, CFL_SIZE);
+	debug_printf(DP_DEBUG2, "nufft kernel dims: ");
+	debug_print_dims(DP_DEBUG2, N, ksp2_dims);
 
-	compute_kern(N, ksp_dims1, ones, bas_dims, basis, wgh_dims, weights);
+	debug_printf(DP_DEBUG2, "nufft psf dims:    ");
+	debug_print_dims(DP_DEBUG2, N, img2_dims);
 
-	complex float* psft = md_alloc(N, img2_dims, CFL_SIZE);
+	debug_printf(DP_DEBUG2, "nufft traj dims:   ");
+	debug_print_dims(DP_DEBUG2, N, trj2_dims);
 
-	linop_adjoint_unchecked(op2, psft, ones);
+	complex float* psft = NULL;
 
-	md_free(ones);
-	linop_free(op2);
+	long pos[N];
+	for (unsigned int i = 0; i < N; i++)
+		pos[i] = 0;
+
+	long A = md_calc_size(N, ksp2_dims);
+	long B = md_calc_size(N - 1, ksp2_dims) + md_calc_size(N - 1, img2_dims);
+	long C = md_calc_size(N, img2_dims);
+
+	if ((A <= B) || !lowmem) {
+
+		debug_printf(DP_DEBUG1, "Allocating %ld (vs. %ld) + %ld\n", A, B, C);
+
+		complex float* ones = md_alloc(N, ksp2_dims, CFL_SIZE);
+
+		compute_kern(N, ~0u, pos, ksp2_dims, ones, bas2_dims, basis, wgh2_dims, weights);
+
+		psft = md_alloc(N, img2_dims, CFL_SIZE);
+
+		struct linop_s* op2 = nufft_create(N, ksp2_dims, img2_dims, trj2_dims, traj, NULL, conf);
+
+		linop_adjoint_unchecked(op2, psft, ones);
+
+		linop_free(op2);
+
+		md_free(ones);
+
+	} else {
+
+		debug_printf(DP_DEBUG1, "Allocating %ld (vs. %ld) + %ld\n", B, A, C);
+
+		psft = md_calloc(N, img2_dims, CFL_SIZE);
+
+		long trj2_strs[N];
+		md_calc_strides(N, trj2_strs, trj2_dims, CFL_SIZE);
+
+		complex float* ones = md_alloc(N - 1, ksp2_dims, CFL_SIZE);
+		complex float* tmp = md_alloc(N - 1, img2_dims, CFL_SIZE);
+
+		assert(!((1 != trj2_dims[N - 1]) && (NULL == basis)));
+
+		for (long i = 0; i < trj2_dims[N - 1]; i++) {
+
+			debug_printf(DP_DEBUG1, "KERN %03ld\n", i);
+
+			unsigned int flags = ~0u;
+
+			if (1 != trj2_dims[N - 1])
+				flags = ~(1u << (N - 1u));
+
+			pos[N - 1] = i;
+			compute_kern(N, flags, pos, ksp2_dims, ones, bas2_dims, basis, wgh2_dims, weights);
+
+			struct linop_s* op2 = nufft_create(N - 1, ksp2_dims, img2_dims, trj2_dims, (void*)traj + i * trj2_strs[N - 1], NULL, conf);
+
+			linop_adjoint_unchecked(op2, tmp, ones);
+			md_zadd(N - 1, img2_dims, psft, psft, tmp);
+
+			linop_free(op2);
+		}
+
+		md_free(ones);
+		md_free(tmp);
+	}
 
 	return psft;
 }
 
 
 static complex float* compute_psf2(int N, const long psf_dims[N + 1], unsigned long flags, const long trj_dims[N + 1], const complex float* traj,
-				const long bas_dims[N + 1], const complex float* basis, const long wgh_dims[N + 1], const complex float* weights, bool periodic)
+				const long bas_dims[N + 1], const complex float* basis, const long wgh_dims[N + 1], const complex float* weights,
+				bool periodic, bool lowmem)
 {
 	int ND = N + 1;
 
@@ -253,7 +366,7 @@ static complex float* compute_psf2(int N, const long psf_dims[N + 1], unsigned l
 	complex float* traj2 = md_alloc(ND, trj_dims, CFL_SIZE);
 	md_zsmul(ND, trj_dims, traj2, traj, 2.);
 
-	complex float* psft = compute_psf(ND, img2_dims, trj_dims, traj2, bas_dims, basis, wgh_dims, weights, periodic);
+	complex float* psft = compute_psf(ND, img2_dims, trj_dims, traj2, bas_dims, basis, wgh_dims, weights, periodic, lowmem);
 	md_free(traj2);
 
 	fftuc(ND, img2_dims, flags, psft, psft);
@@ -324,7 +437,7 @@ struct linop_s* nufft_create2(unsigned int N,
 
 	// dim 0 must be transformed (we treat this special in the trajectory)
 	assert(MD_IS_SET(data->flags, 0));
-	assert(md_check_bounds(N, ~data->flags, ksp_dims, cim_dims));
+//	assert(md_check_compat(N, ~data->flags, ksp_dims, cim_dims));
 	assert(md_check_bounds(N, ~data->flags, cim_dims, ksp_dims));
 
 	// extend internal dimensions by one for linear phases
@@ -372,7 +485,7 @@ struct linop_s* nufft_create2(unsigned int N,
 	long chk_dims[N];
 	md_select_dims(N, ~data->flags, chk_dims, traj_dims);
 	assert(md_check_compat(N, ~0ul, chk_dims, ksp_dims));
-	assert(md_check_bounds(N, ~0ul, chk_dims, ksp_dims));
+//	assert(md_check_bounds(N, ~0ul, chk_dims, ksp_dims));
 
 
 	md_copy_dims(N, data->trj_dims, traj_dims);
@@ -395,7 +508,6 @@ struct linop_s* nufft_create2(unsigned int N,
 
 		data->out_dims[5] = bas_dims[5];	// TE
 		data->out_dims[6] = 1;			// COEFF
-		assert(1 == data->ksp_dims[5]);
 		assert(data->ksp_dims[6] == bas_dims[6]);
 
 		// recompute
@@ -477,15 +589,17 @@ struct linop_s* nufft_create2(unsigned int N,
 
 		if (NULL != basis) {
 
-			assert(1 == data->psf_dims[5]);
-			assert(1 == data->psf_dims[6]);
+			debug_printf(DP_DEBUG3, "psf_dims: ");
+			debug_print_dims(DP_DEBUG3, N, data->psf_dims);
 			data->psf_dims[6] = data->bas_dims[6];
 			data->psf_dims[5] = data->bas_dims[6];
 		}
 
 		md_calc_strides(ND, data->psf_strs, data->psf_dims, CFL_SIZE);
 
-		data->psf = compute_psf2(N, data->psf_dims, data->flags, data->trj_dims, data->traj, data->bas_dims, data->basis, data->wgh_dims, data->weights, true /*conf.periodic*/);
+		data->psf = compute_psf2(N, data->psf_dims, data->flags, data->trj_dims, data->traj,
+					data->bas_dims, data->basis, data->wgh_dims, data->weights,
+					true /*conf.periodic*/, conf.lowmem);
 	}
 
 
@@ -590,6 +704,7 @@ static void nufft_free_data(const linop_data_t* _data)
 	md_free(data->fftmod);
 	md_free(data->weights);
 	md_free(data->roll);
+	md_free(data->basis);
 
 #ifdef USE_CUDA
 	md_free(data->linphase_gpu);
@@ -903,4 +1018,3 @@ void estimate_im_dims(int N, unsigned long flags, long dims[N], const long tdims
 }
 
 
-
