@@ -1,5 +1,5 @@
 /* Copyright 2013-2018. The Regents of the University of California.
- * Copyright 2016-2018. Martin Uecker.
+ * Copyright 2016-2019. Martin Uecker.
  * Copyright 2017. University of Oxford.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
@@ -142,6 +142,20 @@ static double zl1norm(long N, const complex float* vec)
 }
 
 
+
+// we should probably replace asum and zl1norm
+static void zsum(long N, complex float* vec)
+{
+	complex float res = 0.;
+
+	for (long i = 0; i < N; i++)
+		res += vec[i];
+
+	vec[0] = res;
+}
+
+
+
 static void axpbz(long N, float* dst, const float a1, const float* src1, const float a2, const float* src2)
 {
 	for (long i = 0; i < N; i++)
@@ -341,7 +355,7 @@ static void max(long N, float* dst, const float* src1, const float* src2)
 		dst[i] = MAX(src1[i], src2[i]);
 }
 
-static void smax(long N, float* dst, const float* src1, const float val)
+static void smax(long N, float val, float* dst, const float* src1)
 {
 	for (long i = 0; i < N; i++)
 		dst[i] = MAX(src1[i], val);
@@ -355,6 +369,13 @@ static void min(long N, float* dst, const float* src1, const float* src2)
 }
 
 
+static void zsmax(long N, float val, complex float* dst, const complex float* src)
+{
+	for (long i = 0; i < N; i++)
+		dst[i] = MAX(crealf(src[i]), val);
+}
+
+
 static void vec_pow(long N, float* dst, const float* src1, const float* src2)
 {
 	for (long i = 0; i < N; i++)
@@ -576,6 +597,7 @@ const struct vec_ops cpu_ops = {
 	.double2float = double2float,
 	.dot = dot,
 	.asum = asum,
+	.zsum = zsum,
 	.zl1norm = zl1norm,
 
 	.add = add,
@@ -603,6 +625,7 @@ const struct vec_ops cpu_ops = {
 	.zfmacc = zfmacc,
 	.zfmacc2 = zfmacc2,
 
+	.zsmax = zsmax,
 	.zsmul = zsmul,
 
 	.zpow = zpow,
@@ -672,7 +695,7 @@ const struct vec_iter_s cpu_iter_ops = {
 	.add = add,
 	.sub = sub,
 	.swap = swap,
-	.zmul = zmul
+	.zmul = zmul,
 };
 
 
