@@ -20,9 +20,11 @@
 #include "num/flpmath.h"
 #include "num/fft.h"
 #include "num/init.h"
+#include "num/ops_p.h"
 #include "num/ops.h"
 
 #include "iter/misc.h"
+#include "iter/monitor.h"
 
 #include "linops/linop.h"
 #include "linops/fmac.h"
@@ -63,7 +65,7 @@ static const struct linop_s* sense_nc_init(const long max_dims[DIMS], const long
 	long ksp_dims2[DIMS];
 	md_copy_dims(DIMS, ksp_dims2, ksp_dims);
 	ksp_dims2[COEFF_DIM] = max_dims[COEFF_DIM];
-	ksp_dims2[TE_DIM] = 1;
+	//ksp_dims2[TE_DIM] = 1;
 
 	debug_print_dims(DP_INFO, DIMS, ksp_dims2);
 	debug_print_dims(DP_INFO, DIMS, coilim_dims);
@@ -78,21 +80,16 @@ static const struct linop_s* sense_nc_init(const long max_dims[DIMS], const long
 		 * sensitivity weighting but before NUFFT).
 		 */
 
-		const struct linop_s* fft_slice = linop_fft_create(DIMS, map_dims, SLICE_FLAG);
-		const struct linop_s* tmp = fft_op;
-		fft_op = linop_chain(fft_slice, tmp);
-		linop_free(tmp);
-		linop_free(fft_slice);
+		const struct linop_s* fft_slice = linop_fft_create(DIMS, coilim_dims, SLICE_FLAG);
+
+		fft_op = linop_chain_FF(fft_slice, fft_op);
 	}
 
-	const struct linop_s* lop = linop_chain(maps_op, fft_op);
+	const struct linop_s* lop = linop_chain_FF(maps_op, fft_op);
 
 	//precond_op[0] = (struct operator_s*) nufft_precond_create( fft_op );
 	precond_op[0] = NULL;
 
-	linop_free(maps_op);
-	linop_free(fft_op);
-
 	return lop;
 }
 
@@ -118,7 +115,7 @@ int main_pics(int argc, char* argv[])
 	// Read input options
 	struct nufft_conf_s nuconf = nufft_conf_defaults;
 	nuconf.toeplitz = true;
-	nuconf.lowmem = true;
+	nuconf.lowmem = false;
 
 	float restrict_fov = -1.;
 	const char* pat_file = NULL;
@@ -193,6 +190,7 @@ int main_pics(int argc, char* argv[])
 		OPT_FLOAT('P', &bpsense_eps, "eps", "Basis Pursuit formulation, || y- Ax ||_2 <= eps"),
 		OPT_SELECT('a', enum algo_t, &algo, ALGO_PRIDU, "select Primal Dual"),
 		OPT_SET('M', &sms, "Simultaneous Multi-Slice reconstruction"),
+		OPT_SET('U', &nuconf.lowmem, "Use low-mem mode of the nuFFT"),
 	};
 
 	cmdline(&argc, argv, 3, 3, usage_str, help_str, ARRAY_SIZE(opts), opts);
@@ -228,7 +226,6 @@ int main_pics(int argc, char* argv[])
         if (sms) {
 
                 debug_printf(DP_INFO, "SMS reconstruction: MB = %ld\n", ksp_dims[SLICE_DIM]);
-                nuconf.toeplitz = false; // no longer toeplitz-shaped because of chaining of operators (see later)?!
         }
 
 	complex float* maps = load_cfl(argv[2], DIMS, map_dims);
@@ -287,6 +284,11 @@ int main_pics(int argc, char* argv[])
 	if (!md_check_compat(DIMS, ~(MD_BIT(MAPS_DIM)|FFT_FLAGS), img_dims, map_dims))
 		error("Dimensions of image and sensitivities do not match!\n");
 
+	if ((NULL != traj_file) && (!md_check_compat(DIMS, ~0, ksp_dims, traj_dims)))
+		error("Dimensions of data and trajectory do not match!\n");
+
+
+
 	assert(1 == ksp_dims[MAPS_DIM]);
 
 
@@ -355,7 +357,7 @@ int main_pics(int argc, char* argv[])
 
 	if (NULL != traj_file) {
 
-		if (NULL == pat_file) {
+		if (NULL == pat_file && NULL == basis) {
 
 			md_free(pattern);
 			pattern = NULL;
@@ -414,7 +416,7 @@ int main_pics(int argc, char* argv[])
 		if (NULL != basis_file) {
 
 			const struct linop_s* basis_op = linop_fmac_create(DIMS, bmx_dims, COEFF_FLAG, TE_FLAG, ~(COEFF_FLAG | TE_FLAG), basis);
-			forward_op = linop_chain(forward_op, basis_op);
+			forward_op = linop_chain_FF(forward_op, basis_op);
 		}
 
 	} else {
@@ -473,6 +475,14 @@ int main_pics(int argc, char* argv[])
 		image_truth = load_cfl(image_truth_file, DIMS, img_truth_dims);
 		//md_zsmul(DIMS, img_dims, image_truth, image_truth, 1. / scaling);
 
+#ifdef USE_CUDA
+		if (conf.gpu) {
+
+			complex float* gpu_image_truth = md_gpu_move(DIMS, img_dims, image_truth, CFL_SIZE);
+			unmap_cfl(DIMS, img_dims, image_truth);
+			image_truth = gpu_image_truth;
+		}
+#endif
 		xfree(image_truth_file);
 	}
 
@@ -496,6 +506,7 @@ int main_pics(int argc, char* argv[])
 
 
 
+	assert((0u == loop_flags) || (NULL == image_truth));
 	assert((0u == loop_flags) || (NULL == image_start));
 	assert((0u == loop_flags) || (NULL == traj_file));
 	assert(!(loop_flags & COIL_FLAG));
@@ -612,11 +623,19 @@ int main_pics(int argc, char* argv[])
 			    || (ALGO_ADMM == algo)
 			    || (   (ALGO_NIHT == algo)
 				&& (regs[0].xform == NIHTWAV)));
+
+	// FIXME: will fail with looped dims
+	struct iter_monitor_s* monitor = NULL;
+	if (im_truth)
+		monitor = create_monitor(2*md_calc_size(DIMS, img_dims), (const float*)image_truth, NULL, NULL); 
 	
-	const struct operator_s* op = sense_recon_create(&conf, max1_dims, forward_op,
+	const struct operator_p_s* po = sense_recon_create(&conf, max1_dims, forward_op,
 				pat1_dims, ((NULL != traj_file) || conf.bpsense) ? NULL : pattern1,
 				it.italgo, it.iconf, image_start1, nr_penalties, thresh_ops,
-				trafos_cond ? trafos : NULL, precond_op);
+				trafos_cond ? trafos : NULL, precond_op, monitor);
+
+	const struct operator_s* op = operator_p_bind(po, 1.);
+//	operator_p_free(po);	// FIXME
 
 	long strsx[2][DIMS];
 	const long* strs[2] = { strsx[0], strsx[1] };
@@ -669,14 +688,22 @@ int main_pics(int argc, char* argv[])
 	if (NULL != traj)
 		unmap_cfl(DIMS, traj_dims, traj);
 
-	if (im_truth)
-		unmap_cfl(DIMS, img_dims, image_truth);
+	if (im_truth) {
+
+#ifdef USE_CUDA
+		if (conf.gpu)
+			md_free(image_truth);
+		else
+#endif
+			unmap_cfl(DIMS, img_dims, image_truth);
+	}
 
 	if (image_start)
 		unmap_cfl(DIMS, img_dims, image_start);
 
 	xfree(pat_file);
 	xfree(traj_file);
+	xfree(basis_file);
 
 	double end_time = timestamp();
 
