@@ -72,7 +72,7 @@ int main_reshape(int argc, char* argv[])
 
 	unmap_cfl(DIMS, in_dims, in_data);
 	unmap_cfl(DIMS, out_dims, out_data);
-	exit(0);
+	return 0;
 }
 
 
