@@ -76,7 +76,7 @@ int main_resize(int argc, char* argv[])
 	unmap_cfl(N, in_dims, in_data);
 	unmap_cfl(N, out_dims, out_data);
 
-	exit(0);
+	return 0;
 }
 
 
