@@ -49,7 +49,7 @@ int main_window(int argc, char* argv[])
 
 	unmap_cfl(DIMS, dims, in_data);
 	unmap_cfl(DIMS, dims, out_data);
-	exit(0);
+	return 0;
 }
 
 
