@@ -1,9 +1,9 @@
-/* Copyright 2018. Massachusetts Institute of Technology.
+/* Copyright 2018-2019. Massachusetts Institute of Technology.
  * All rights reserved. Use of this source code is governed by
  * a BSD-style license which can be found in the LICENSE file.
  *
  * Authors: 
- * 2018 Siddharth Iyer <ssi@mit.edu>
+ * 2018-2019 Siddharth Iyer <ssi@mit.edu>
  *
  * Tamir J, Uecker M, Chen W, Lai P, Alley MT, Vasanawala SS, Lustig M. 
  * T2 shuffling: Sharp, multicontrast, volumetric fast spin‐echo imaging. 
@@ -19,7 +19,6 @@
  * https://www.ismrm.org/18/program_files/O67.htm 
  */
 
-#include <assert.h>
 #include <stdbool.h>
 #include <complex.h>
 #include <math.h>
@@ -33,6 +32,7 @@
 #include "num/init.h"
 #include "num/iovec.h"
 #include "num/ops.h"
+#include "num/ops_p.h"
 #ifdef USE_CUDA
 #include "num/gpuops.h"
 #endif
@@ -44,7 +44,7 @@
 #include "linops/linop.h"
 #include "linops/fmac.h"
 #include "linops/someops.h"
-#include "linops/realval.h"
+#include "linops/decompose_complex.h"
 
 #include "misc/debug.h"
 #include "misc/mri.h"
@@ -97,10 +97,14 @@ static void print_opdims(const struct linop_s* op)
 {
 	const struct iovec_s* domain   = linop_domain(op);
 	const struct iovec_s* codomain = linop_codomain(op);
-	debug_printf(DP_INFO, "\tDomain:   ");
-	debug_print_dims(DP_INFO, domain->N, domain->dims);
-	debug_printf(DP_INFO, "\tCodomain: ");
-	debug_print_dims(DP_INFO, codomain->N, codomain->dims);
+	debug_printf(DP_INFO, "\tDomain:   [");
+	for (long k = 0; k < domain->N; k ++)
+		debug_printf(DP_INFO, "%6ld", domain->dims[k]);
+	debug_printf(DP_INFO, "]\n");
+	debug_printf(DP_INFO, "\tCodomain: [");
+	for (long k = 0; k < codomain->N; k ++)
+		debug_printf(DP_INFO, "%6ld", codomain->dims[k]);
+	debug_printf(DP_INFO, "]\n");
 }
 
 /* Construct sampling mask array from reorder tables. */
@@ -124,9 +128,9 @@ static void construct_mask(
 	}
 }
 
-static DEF_TYPEID(kern_s);
 
 struct kern_s {
+
 	INTERFACE(linop_data_t);
 
 	unsigned int N;
@@ -143,6 +147,8 @@ struct kern_s {
 	complex float* gpu_kernel;
 };
 
+static DEF_TYPEID(kern_s);
+
 /* Go to table from coefficient-kspace with memory efficiency. */
 static void kern_apply(const linop_data_t* _data, complex float* dst, const complex float* src)
 {
@@ -247,7 +253,8 @@ static void kern_adjoint(const linop_data_t* _data, complex float* dst, const co
 	long line_dims[]    = {wx, nc,  1,  1};
 	long vthrd_dims[]   = {wx, nc, tf,  1, num_threads};
 
-	complex float* vec = md_calloc(5, vthrd_dims, CFL_SIZE);
+	complex float* vec = md_alloc_sameplace(5, vthrd_dims, CFL_SIZE, dst);
+	md_clear(DIMS, vthrd_dims, vec, CFL_SIZE);
 
 	long vec_str[4];
 	md_calc_strides(4, vec_str, vec_dims, CFL_SIZE);
@@ -258,27 +265,33 @@ static void kern_adjoint(const linop_data_t* _data, complex float* dst, const co
 	long fmac_str[4];
 	md_calc_strides(4, fmac_str, fmac_dims, CFL_SIZE);
 
+	long flag_dims[1] = { n };
+	complex float* flags = md_calloc(1, flag_dims, CFL_SIZE);
+
 	#pragma omp parallel for
-	for (int k = 0; k < sy * sz; k ++) {
+	for (int k = 0; k < n; k ++) {
 #ifdef _OPENMP
 		int tid = omp_get_thread_num();
 #else
 		int tid = 0;
 #endif
-		int y = k % sy;
-		int z = k / sy;
+		int y = lround(creal(data->reorder[k]));
+		int z = lround(creal(data->reorder[k + n]));
 		int t = -1;
 
-		md_clear(4, vec_dims, vec + (wx * nc * tf * tid), CFL_SIZE);
+		if (0 == flags[k]) {
+			md_clear(4, vec_dims, vec + (wx * nc * tf * tid), CFL_SIZE);
 
-		for (int i = 0; i < n; i ++) {
-			if ((y == lround(creal(data->reorder[i]))) && (z == lround(creal(data->reorder[i + n])))) {
-				t = lround(creal(data->reorder[i + 2 * n]));
-				md_copy(4, line_dims, (vec + (wx * nc * tf * tid) + t * wx * nc), (src + i * wx * nc), CFL_SIZE);
+			for (int i = k; i < n; i ++) {
+				if ((y == lround(creal(data->reorder[i]))) && (z == lround(creal(data->reorder[i + n])))) {
+					flags[i] = 1;
+					t = lround(creal(data->reorder[i + 2 * n]));
+					md_copy(4, line_dims, (vec + (wx * nc * tf * tid) + t * wx * nc), (src + i * wx * nc), CFL_SIZE);
+				}
 			}
-		}
 
-		md_zfmacc2(4, fmac_dims, phi_out_str, perm + (y + z * sy) * (wx * nc * tk), vec_str, vec + (wx * nc * tf * tid), phi_mat_str, data->phi);
+			md_zfmacc2(4, fmac_dims, phi_out_str, perm + (y + z * sy) * (wx * nc * tk), vec_str, vec + (wx * nc * tf * tid), phi_mat_str, data->phi);
+		}
 	}
 
 	long out_dims[] = { [0 ... DIMS - 1] = 1 };
@@ -294,6 +307,7 @@ static void kern_adjoint(const linop_data_t* _data, complex float* dst, const co
 
 	md_free(vec);
 	md_free(perm);
+	md_free(flags);
 }
 
 static void kern_normal(const linop_data_t* _data, complex float* dst, const complex float* src)
@@ -371,8 +385,6 @@ static const struct linop_s* linop_kern_create(bool gpu_flag,
 	PTR_ALLOC(struct kern_s, data);
 	SET_TYPEID(kern_s, data);
 
-	data->N = DIMS;
-
 	PTR_ALLOC(long[DIMS], reorder_dims);
 	PTR_ALLOC(long[DIMS], phi_dims);
 	PTR_ALLOC(long[DIMS], table_dims);
@@ -433,24 +445,271 @@ static const struct linop_s* linop_kern_create(bool gpu_flag,
 	return K;
 }
 
-/* ESPIRiT operator. */
-static const struct linop_s* linop_espirit_create(long sx, long sy, long sz, long nc, long md, long tk, complex float* maps)
-{
-	long max_dims[] = { [0 ... DIMS - 1] = 1};
-	max_dims[0] = sx;
-	max_dims[1] = sy;
-	max_dims[2] = sz;
-	max_dims[3] = nc;
-	max_dims[4] = md;
-	max_dims[6] = tk;
+struct multc_s {
+	INTERFACE(linop_data_t);
 
-	const struct linop_s* E = linop_fmac_create(DIMS, max_dims, MAPS_FLAG, COIL_FLAG, TE_FLAG|COEFF_FLAG, maps);
- 
+	unsigned int nc;
+	unsigned int md;
+	const complex float* maps;
+	const struct linop_s* sc_op; // Single channel operator.
+};
+
+static DEF_TYPEID(multc_s);
+
+static void multc_apply(const linop_data_t* _data, complex float* dst, const complex float* src)
+{
+	const struct multc_s* data = CAST_DOWN(multc_s, _data);
+
+	// Loading single channel operator.
+	const struct operator_s* fwd = data->sc_op->forward;
+	const long* sc_inp_dims = linop_domain(data->sc_op)->dims;
+	const long* sc_out_dims = linop_codomain(data->sc_op)->dims;
+
+	long sx = sc_inp_dims[0];
+	long sy = sc_inp_dims[1];
+	long sz = sc_inp_dims[2];
+	long wx = sc_out_dims[0];
+	long  n = sc_out_dims[2];
+	long nc = data->nc;
+	long md = data->md;
+
+	long src_dims[] = { [0 ... DIMS - 1] = 1};
+	md_copy_dims(DIMS, src_dims, sc_inp_dims);
+	src_dims[MAPS_DIM] = md;
+
+	long dst_dims[] = { [0 ... DIMS - 1] = 1};
+	md_copy_dims(DIMS, dst_dims, sc_out_dims);
+	dst_dims[1] = nc;
+
+	long map_dims[] = { [0 ... DIMS - 1] = 1};
+	map_dims[0] = sx;
+	map_dims[1] = sy;
+	map_dims[2] = sz;
+	map_dims[3] = nc;
+	map_dims[4] = md;
+
+	long single_map_dims[] = { [0 ... DIMS - 1] = 1 };
+	md_copy_dims(DIMS, single_map_dims, map_dims);
+	single_map_dims[COIL_DIM] = 1;
+	complex float* single_map = md_alloc_sameplace(DIMS, single_map_dims, CFL_SIZE, src);
+
+	complex float* buffer = md_alloc_sameplace(DIMS, sc_inp_dims, CFL_SIZE, src);
+
+	long tbl_dims[] = { [0 ... DIMS - 1] = 1};
+	tbl_dims[0] = wx;
+	tbl_dims[1] = n;
+	tbl_dims[2] = nc;
+	complex float* tbl = md_alloc_sameplace(DIMS, tbl_dims, CFL_SIZE, src);
+	md_clear(DIMS, tbl_dims, tbl, CFL_SIZE);
+
+	long pos[] = { [0 ... DIMS - 1] = 0 };
+
+	long zfmac_dims[] = { [0 ... DIMS - 1] = 1 };
+	md_copy_dims(DIMS, zfmac_dims, src_dims);
+
+	long strides_single_map[DIMS];
+	md_calc_strides(DIMS, strides_single_map, single_map_dims, CFL_SIZE);
+	long strides_src[DIMS];
+	md_calc_strides(DIMS, strides_src, src_dims, CFL_SIZE);
+	long strides_sc_inp[DIMS];
+	md_calc_strides(DIMS, strides_sc_inp, sc_inp_dims, CFL_SIZE);
+
+	for (long k = 0; k < data->nc; k++) {
+		md_clear(DIMS, single_map_dims, single_map, CFL_SIZE);
+		md_clear(DIMS, sc_inp_dims, buffer, CFL_SIZE);
+		pos[COIL_DIM] = k;
+		md_slice(DIMS, COIL_FLAG, pos, map_dims, single_map, data->maps, CFL_SIZE);
+		pos[COIL_DIM] = 0;
+		md_zfmac2(DIMS, zfmac_dims, strides_sc_inp, buffer, strides_src, src, strides_single_map, single_map);
+		operator_apply(fwd, DIMS, sc_out_dims, tbl  + (wx * n * k), DIMS, sc_inp_dims, buffer);
+	}
+
+	md_clear(DIMS, dst_dims, dst, CFL_SIZE);
+	unsigned int permute_order[DIMS] = {0, 2, 1};
+	for (unsigned int i = 3; i < DIMS; i++)
+		permute_order[i] = i;
+	md_permute(DIMS, permute_order, dst_dims, dst, tbl_dims, tbl, CFL_SIZE);
+
+	md_free(single_map);
+	md_free(buffer);
+	md_free(tbl);
+}
+
+static void multc_adjoint(const linop_data_t* _data, complex float* dst, const complex float* src)
+{
+	const struct multc_s* data = CAST_DOWN(multc_s, _data);
+
+	// Loading single channel operator.
+	const struct operator_s* adj = data->sc_op->adjoint;
+	const long* sc_inp_dims = linop_codomain(data->sc_op)->dims;
+	const long* sc_out_dims = linop_domain(data->sc_op)->dims;
+
+	long sx = sc_out_dims[0];
+	long sy = sc_out_dims[1];
+	long sz = sc_out_dims[2];
+	long wx = sc_inp_dims[0];
+	long  n = sc_inp_dims[2];
+	long nc = data->nc;
+	long md = data->md;
+
+	long src_dims[] = { [0 ... DIMS - 1] = 1};
+	md_copy_dims(DIMS, src_dims, sc_inp_dims);
+	src_dims[1] = nc;
+
+	long dst_dims[] = { [0 ... DIMS - 1] = 1};
+	md_copy_dims(DIMS, dst_dims, sc_out_dims);
+	dst_dims[MAPS_DIM] = md;
+
+	long map_dims[] = { [0 ... DIMS - 1] = 1};
+	map_dims[0] = sx;
+	map_dims[1] = sy;
+	map_dims[2] = sz;
+	map_dims[3] = nc;
+	map_dims[4] = md;
+
+	long single_map_dims[] = { [0 ... DIMS - 1] = 1 };
+	md_copy_dims(DIMS, single_map_dims, map_dims);
+	single_map_dims[COIL_DIM] = 1;
+	complex float* single_map = md_alloc_sameplace(DIMS, single_map_dims, CFL_SIZE, src);
+
+	complex float* buffer1 = md_alloc_sameplace(DIMS, sc_out_dims, CFL_SIZE, src);
+	complex float* buffer2 = md_alloc_sameplace(DIMS, dst_dims, CFL_SIZE, src);
+
+	long tbl_dims[] = { [0 ... DIMS - 1] = 1};
+	tbl_dims[0] = wx;
+	tbl_dims[2] = n;
+	complex float* tbl = md_alloc_sameplace(DIMS, tbl_dims, CFL_SIZE, src);
+
+	long pos[] = { [0 ... DIMS - 1] = 0 };
+
+	long strides_single_map[DIMS];
+	md_calc_strides(DIMS, strides_single_map, single_map_dims, CFL_SIZE);
+	long strides_sc_out[DIMS];
+	md_calc_strides(DIMS, strides_sc_out, sc_out_dims, CFL_SIZE);
+	long strides_dst[DIMS];
+	md_calc_strides(DIMS, strides_dst, dst_dims, CFL_SIZE);
+
+	md_clear(DIMS, dst_dims, dst, CFL_SIZE);
+	
+	for (long k = 0; k < data->nc; k++) {
+		md_clear(DIMS, single_map_dims, single_map, CFL_SIZE);
+		md_clear(DIMS, sc_out_dims, buffer1, CFL_SIZE);
+		md_clear(DIMS, dst_dims, buffer2, CFL_SIZE);
+		md_clear(DIMS, tbl_dims, tbl, CFL_SIZE);
+		pos[1] = k;
+		md_slice(DIMS, 2, pos, src_dims, tbl, src, CFL_SIZE);
+		pos[1] = 0;
+		operator_apply(adj, DIMS, sc_out_dims, buffer1, DIMS, tbl_dims, tbl);
+		pos[COIL_DIM] = k;
+		md_slice(DIMS, COIL_FLAG, pos, map_dims, single_map, data->maps, CFL_SIZE);
+		pos[COIL_DIM] = 0;
+		md_zfmacc2(DIMS, dst_dims, strides_dst, buffer2, strides_sc_out, buffer1, strides_single_map, single_map);
+		md_zadd(DIMS, dst_dims, dst, dst, buffer2);
+	}
+
+	md_free(single_map);
+	md_free(buffer1);
+	md_free(buffer2);
+	md_free(tbl);
+}
+
+static void multc_normal(const linop_data_t* _data, complex float* dst, const complex float* src)
+{
+	const struct multc_s* data = CAST_DOWN(multc_s, _data);
+
+	// Loading single channel operator.
+	const struct operator_s* nrm = data->sc_op->normal;
+	const long* sc_dims = linop_domain(data->sc_op)->dims;
+
+	long sx = sc_dims[0];
+	long sy = sc_dims[1];
+	long sz = sc_dims[2];
+	long nc = data->nc;
+	long md = data->md;
+
+	long dims[] = { [0 ... DIMS - 1] = 1};
+	md_copy_dims(DIMS, dims, sc_dims);
+	dims[MAPS_DIM] = md;
+
+	long map_dims[] = { [0 ... DIMS - 1] = 1};
+	map_dims[0] = sx;
+	map_dims[1] = sy;
+	map_dims[2] = sz;
+	map_dims[3] = nc;
+	map_dims[4] = md;
+
+	long single_map_dims[] = { [0 ... DIMS - 1] = 1 };
+	md_copy_dims(DIMS, single_map_dims, map_dims);
+	single_map_dims[COIL_DIM] = 1;
+	complex float* single_map = md_alloc_sameplace(DIMS, single_map_dims, CFL_SIZE, src);
+
+	complex float* buffer1 = md_alloc_sameplace(DIMS, sc_dims, CFL_SIZE, src);
+	complex float* buffer2 = md_alloc_sameplace(DIMS, sc_dims, CFL_SIZE, src);
+	complex float* buffer3 = md_alloc_sameplace(DIMS, dims, CFL_SIZE, src);
+
+	long pos[] = { [0 ... DIMS - 1] = 0 };
+
+	long strides_single_map[DIMS];
+	md_calc_strides(DIMS, strides_single_map, single_map_dims, CFL_SIZE);
+	long strides_sc[DIMS];
+	md_calc_strides(DIMS, strides_sc, sc_dims, CFL_SIZE);
+	long strides[DIMS];
+	md_calc_strides(DIMS, strides, dims, CFL_SIZE);
+
+	md_clear(DIMS, dims, dst, CFL_SIZE);
+	for (long k = 0; k < data->nc; k++) {
+		md_clear(DIMS, single_map_dims, single_map, CFL_SIZE);
+		md_clear(DIMS, sc_dims, buffer1, CFL_SIZE);
+		md_clear(DIMS, sc_dims, buffer2, CFL_SIZE);
+		md_clear(DIMS, dims, buffer3, CFL_SIZE);
+		pos[COIL_DIM] = k;
+		md_slice(DIMS, COIL_FLAG, pos, map_dims, single_map, data->maps, CFL_SIZE);
+		pos[COIL_DIM] = 0;
+		md_zfmac2(DIMS, dims, strides_sc, buffer1, strides, src, strides_single_map, single_map);
+		operator_apply(nrm, DIMS, sc_dims, buffer2, DIMS, sc_dims, buffer1);
+		md_zfmacc2(DIMS, dims, strides, buffer3, strides_sc, buffer2, strides_single_map, single_map);
+		md_zadd(DIMS, dims, dst, dst, buffer3);
+	}
+
+	md_free(single_map);
+	md_free(buffer1);
+	md_free(buffer2);
+	md_free(buffer3);
+}
+
+static void multc_free(const linop_data_t* _data)
+{
+	const struct multc_s* data = CAST_DOWN(multc_s, _data);
+	xfree(data);
+}
+
+static struct linop_s* linop_multc_create(long nc, long md, const complex float* maps, const struct linop_s* sc_op)
+{
+	PTR_ALLOC(struct multc_s, data);
+	SET_TYPEID(multc_s, data);
+
+	data->nc = nc;
+	data->md = md;
+	data->maps = maps;
+	data->sc_op = sc_op;
+
+	long* op_inp_dims = (long*) linop_domain(sc_op)->dims;
+	long* op_out_dims = (long*) linop_codomain(sc_op)->dims;
+
+	long input_dims[] = { [0 ... DIMS - 1] = 1 };
+	md_copy_dims(DIMS, input_dims, op_inp_dims);
+	input_dims[MAPS_DIM] = md;
+
+	long output_dims[] = { [0 ... DIMS - 1] = 1 };
+	md_copy_dims(DIMS, output_dims, op_out_dims);
+	output_dims[1] = nc;
+
+	struct linop_s* E = linop_create(DIMS, output_dims, DIMS, input_dims, CAST_UP(PTR_PASS(data)), multc_apply, multc_adjoint, multc_normal, NULL, multc_free);
 	return E;
 }
 
 /* Resize operator. */
-static const struct linop_s* linop_reshape_create(long wx, long sx, long sy, long sz, long nc, long tk)
+static const struct linop_s* linop_wavereshape_create(long wx, long sx, long sy, long sz, long nc, long tk)
 {
 	long input_dims[] = { [0 ... DIMS - 1] = 1};
 	input_dims[0] = sx;
@@ -483,7 +742,7 @@ static const struct linop_s* linop_fx_create(long wx, long sy, long sz, long nc,
 }
 
 /* Wave operator. */
-static const struct linop_s* linop_wave_create(long wx, long sy, long sz, long nc, long tk, complex float* psf)
+static const struct linop_s* linop_wave_create(long wx, long sy, long sz, long nc, long tk, long psf_tk, complex float* psf)
 {
 	long dims[] = { [0 ... DIMS - 1] = 1};
 	dims[0] = wx;
@@ -491,8 +750,7 @@ static const struct linop_s* linop_wave_create(long wx, long sy, long sz, long n
 	dims[2] = sz;
 	dims[3] = nc;
 	dims[6] = tk;
-	struct linop_s* W = linop_cdiag_create(DIMS, dims, FFT_FLAGS, psf);
-	return W;
+	return (psf_tk > 1) ? linop_cdiag_create(DIMS, dims, FFT_FLAGS | COEFF_FLAG, psf) : linop_cdiag_create(DIMS, dims, FFT_FLAGS, psf);
 }
 
 /* Fyz operator. */
@@ -637,7 +895,8 @@ int main_wshfl(int argc, char* argv[])
 	const char* fwd = NULL;
 	const char* x0  = NULL;
 	int   gpun      = -1;
-	bool  rvc       = false;
+	bool  dcx       = false;
+	bool  pf        = false;
 
 	const struct opt_s opts[] = {
 		OPT_FLOAT( 'r', &lambda,  "lambda", "Soft threshold lambda for wavelet or locally low rank."),
@@ -652,13 +911,17 @@ int main_wshfl(int argc, char* argv[])
 		OPT_INT(   'g', &gpun,    "gpunm",  "GPU device number."),
 		OPT_SET(   'f', &fista,             "Reconstruct using FISTA instead of IST."),
 		OPT_SET(   'H', &hgwld,             "Use hogwild in IST/FISTA."),
+		OPT_SET(   'v', &dcx,               "Split coefficients to real and imaginary components."),
 		OPT_SET(   'w', &wav,               "Use wavelet."),
-		OPT_SET(   'l', &llr,               "Use locally low rank."),
-		OPT_SET(   'v', &rvc,               "Apply real valued constraint on coefficients."),
+		OPT_SET(   'l', &llr,               "Use locally low rank across temporal coefficients."),
+		OPT_SET(   'p', &pf,                "Use locally low rank and real-imaginary components for partial fourier."),
 	};
 
 	cmdline(&argc, argv, 6, 6, usage_str, help_str, ARRAY_SIZE(opts), opts);
 
+	if (pf)
+		dcx = true;
+
 	debug_printf(DP_INFO, "Loading data... ");
 
 	long maps_dims[DIMS];
@@ -718,58 +981,78 @@ int main_wshfl(int argc, char* argv[])
 	coeff_dims[2] = sz;
 	coeff_dims[4] = md;
 	coeff_dims[6] = tk;
+	coeff_dims[8] = dcx ? 2 : 1;
 
-	debug_printf(DP_INFO, "Creating linear operators:\n");
+	debug_printf(DP_INFO, "Creating single channel linear operators:\n");
 
 	double t1;
 	double t2;
 
 	t1 = timestamp();
-	const struct linop_s* E = linop_espirit_create(sx, sy, sz, nc, md, tk, maps);
-	t2 = timestamp();
-	debug_printf(DP_INFO, "\tE:   %f seconds.\n", t2 - t1);
-
-	t1 = timestamp();
-	const struct linop_s* R = linop_reshape_create(wx, sx, sy, sz, nc, tk);
+	const struct linop_s* R = linop_wavereshape_create(wx, sx, sy, sz, 1, tk);
 	t2 = timestamp();
 	debug_printf(DP_INFO, "\tR:   %f seconds.\n", t2 - t1);
 
 	t1 = timestamp();
-	const struct linop_s* Fx = linop_fx_create(wx, sy, sz, nc, tk, false);
+	const struct linop_s* Fx = linop_fx_create(wx, sy, sz, 1, tk, false);
 	t2 = timestamp();
 	debug_printf(DP_INFO, "\tFx:  %f seconds.\n", t2 - t1);
 
 	t1 = timestamp();
-	const struct linop_s* W = linop_wave_create(wx, sy, sz, nc, tk, wave);
+	const struct linop_s* W = linop_wave_create(wx, sy, sz, 1, tk, wave_dims[COEFF_DIM], wave);
 	t2 = timestamp();
 	debug_printf(DP_INFO, "\tW:   %f seconds.\n", t2 - t1);
 
 	t1 = timestamp();
-	const struct linop_s* Fyz = linop_fyz_create(wx, sy, sz, nc, tk, false);
+	const struct linop_s* Fyz = linop_fyz_create(wx, sy, sz, 1, tk, false);
 	t2 = timestamp();
 	debug_printf(DP_INFO, "\tFyz: %f seconds.\n", t2 - t1);
 
 	t1 = timestamp();
-	const struct linop_s* K = linop_kern_create(gpun >= 0, reorder_dims, reorder, phi_dims, phi, kernel_dims, kernel, table_dims);
+	long single_channel_table_dims[] = { [0 ... DIMS - 1] = 1 };
+	md_copy_dims(DIMS, single_channel_table_dims, table_dims);
+	single_channel_table_dims[1] = 1;
+	const struct linop_s* K = linop_kern_create(gpun >= 0, reorder_dims, reorder, phi_dims, phi, kernel_dims, kernel, single_channel_table_dims);
 	t2 = timestamp();
 	debug_printf(DP_INFO, "\tK:   %f seconds.\n", t2 - t1);
 
+	struct linop_s* A_sc = linop_chain_FF(linop_chain_FF(linop_chain_FF(linop_chain_FF(
+		R, Fx), W), Fyz), K);
+
+	debug_printf(DP_INFO, "Single channel forward operator information:\n");
+	print_opdims(A_sc);
+	if (eval < 0)	
+#ifdef USE_CUDA
+		eval = (gpun >= 0) ? estimate_maxeigenval_gpu(A_sc->normal) : estimate_maxeigenval(A_sc->normal);
+#else
+		eval = estimate_maxeigenval(A_sc->normal);
+#endif
+	debug_printf(DP_INFO, "\tMax eval: %.2e\n", eval);
+	step /= eval;
+
+	struct linop_s* A = linop_multc_create(nc, md, maps, A_sc);
+	debug_printf(DP_INFO, "Overall forward linear operator information:\n");
+	print_opdims(A);
+
 	if (fwd != NULL) {
 
 		debug_printf(DP_INFO, "Going from coefficients to data table... ");
 		complex float* coeffs_to_fwd = load_cfl(fwd, DIMS, coeff_dims);
 		complex float* table_forward = create_cfl(argv[6], DIMS, table_dims);
-		const struct linop_s* CFx    = linop_fx_create( wx, sy, sz, nc, tk, true);
-		const struct linop_s* CFyz   = linop_fyz_create(wx, sy, sz, nc, tk, true);
-		struct linop_s* AC = linop_chain_FF(linop_chain_FF(linop_chain_FF(linop_chain_FF(linop_chain_FF(
-			E, R), CFx), W), CFyz), K);
+		const struct linop_s* R      = linop_wavereshape_create(wx, sx, sy, sz, 1, tk);
+		const struct linop_s* CFx    = linop_fx_create( wx, sy, sz, 1, tk, true);
+		const struct linop_s* W      = linop_wave_create(wx, sy, sz, 1, tk, wave_dims[COEFF_DIM], wave);
+		const struct linop_s* CFyz   = linop_fyz_create(wx, sy, sz, 1, tk, true);
+		const struct linop_s* K      = linop_kern_create(gpun >= 0, reorder_dims, reorder, phi_dims, phi, kernel_dims, kernel, single_channel_table_dims);
+		struct linop_s* AC_sc = linop_chain_FF(linop_chain_FF(linop_chain_FF(linop_chain_FF(
+			R, CFx), W), CFyz), K);
+		struct linop_s* AC = linop_multc_create(nc, md, maps, AC_sc);
 		operator_apply(AC->forward, DIMS, table_dims, table_forward, DIMS, coeff_dims, coeffs_to_fwd);
 		debug_printf(DP_INFO, "Done.\n");
 
 		debug_printf(DP_INFO, "Cleaning up... ");
-		linop_free(Fx);
-		linop_free(CFyz);
 		linop_free(AC);
+		linop_free(AC_sc);
 		md_free(kernel);
 		unmap_cfl(DIMS, maps_dims, maps);
 		unmap_cfl(DIMS, wave_dims, wave);
@@ -782,32 +1065,17 @@ int main_wshfl(int argc, char* argv[])
 		return 0;
 	}
 
-	debug_printf(DP_INFO, "Forward linear operator information:\n");
-	struct linop_s* A = linop_chain_FF(linop_chain_FF(linop_chain_FF(linop_chain_FF(linop_chain_FF(
-		E, R), Fx), W), Fyz), K);
-
-	if (rvc == true) {
-		debug_printf(DP_INFO, "\tDomain is restricted to real numbers.\n");
+	if (dcx) {
+		debug_printf(DP_INFO, "\tSplitting result into real and imaginary components.\n");
 		struct linop_s* tmp = A;
-		struct linop_s* rvcop = linop_realval_create(DIMS, linop_domain(A)->dims);
+		struct linop_s* dcxop = linop_decompose_complex_create(DIMS, ITER_DIM, linop_domain(A)->dims);
 
-		A = linop_chain(rvcop, tmp);
+		A = linop_chain(dcxop, tmp);
 
-		linop_free(rvcop);
+		linop_free(dcxop);
 		linop_free(tmp);
 	}
 
-	print_opdims(A);
-
-	if (eval < 0)	
-#ifdef USE_CUDA
-		eval = (gpun >= 0) ? estimate_maxeigenval_gpu(A->normal) : estimate_maxeigenval(A->normal);
-#else
-		eval = estimate_maxeigenval(A->normal);
-#endif
-	debug_printf(DP_INFO, "\tMax eval: %.2e\n", eval);
-	step /= eval;
-
 	debug_printf(DP_INFO, "Normalizing data table and applying fftmod to table and maps... ");
 	float norm = md_znorm(DIMS, table_dims, table);
 	md_zsmul(DIMS, table_dims, table, table, 1. / norm);
@@ -823,15 +1091,20 @@ int main_wshfl(int argc, char* argv[])
 	unsigned int WAVFLAG = (sx > 1) * READ_FLAG | (sy > 1) * PHS1_FLAG | (sz > 2) * PHS2_FLAG;
 
 	enum algo_t algo = CG;
-	if ((wav == true) || (llr == true)) {
+	if ((wav) || (llr) || (pf)) {
 		algo = (fista) ? FISTA : IST;
 		if (wav) {
 			debug_printf(DP_INFO, "Creating wavelet threshold operator... ");
 			T = prox_wavelet_thresh_create(DIMS, coeff_dims, WAVFLAG, 0u, minsize, lambda, true);
+		} else if (llr) {
+			debug_printf(DP_INFO, "Creating locally low rank threshold operator across coeff and real-imag... ");
+			llr_blkdims(blkdims, ~(COEFF_FLAG | ITER_FLAG), coeff_dims, blksize);
+			T = lrthresh_create(coeff_dims, true, ~(COEFF_FLAG | ITER_FLAG), (const long (*)[])blkdims, lambda, false, false, false);
 		} else {
-			debug_printf(DP_INFO, "Creating locally low rank threshold operator... ");
-			llr_blkdims(blkdims, ~COEFF_DIM, coeff_dims, blksize);
-			T = lrthresh_create(coeff_dims, true, ~COEFF_FLAG, (const long (*)[])blkdims, lambda, false, false, false);
+			assert(dcx);
+			debug_printf(DP_INFO, "Creating locally low rank threshold operator across real-imag... ");
+			llr_blkdims(blkdims, ~ITER_FLAG, coeff_dims, blksize);
+			T = lrthresh_create(coeff_dims, true, ~ITER_FLAG, (const long (*)[])blkdims, lambda, false, false, false);
 		}
 		debug_printf(DP_INFO, "Done.\n");
 	}
@@ -921,14 +1194,15 @@ int main_wshfl(int argc, char* argv[])
 	complex float* recon = create_cfl(argv[6], DIMS, coeff_dims);
 	struct lsqr_conf lsqr_conf = { 0., gpun >= 0 };
 	double recon_start = timestamp();
-	const struct operator_s* J = lsqr2_create(&lsqr_conf, italgo, iconf, (const float*) init, A, NULL, 1, &T, NULL, NULL);
-	operator_apply(J, DIMS, coeff_dims, recon, DIMS, table_dims, table);
+	const struct operator_p_s* J = lsqr2_create(&lsqr_conf, italgo, iconf, (const float*) init, A, NULL, 1, &T, NULL, NULL);
+	operator_p_apply(J, 1., DIMS, coeff_dims, recon, DIMS, table_dims, table);
 	double recon_end = timestamp();
 	debug_printf(DP_INFO, "Done.\nReconstruction time: %f seconds.\n", recon_end - recon_start);
 
 	debug_printf(DP_INFO, "Cleaning up and saving result... ");
-	operator_free(J);
+	operator_p_free(J);
 	linop_free(A);
+	linop_free(A_sc);
 	md_free(kernel);
 	unmap_cfl(DIMS, maps_dims, maps);
 	unmap_cfl(DIMS, wave_dims, wave);
